/*
 *      quickvpn.c
 *      
 *      Copyright 2008 Fabio DM <pobfdm@gmail.com>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <gtk/gtk.h>
#include <signal.h>
#include <gio/gio.h>

#include <dirent.h>
#include "message.c"


//For Management server
#include<sys/types.h>
#include<sys/socket.h>
#include<netdb.h>
#include<netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#define MAXBUF 8000



#define VERSION "QuickVPN 0.4.0"
#define TERM "xterm -T 'OpenVpn output:' -bg white -fg black -e "
#define LICENSE "This program is free software; you can redistribute it and/or modify \n \
it under the terms of the GNU General Public License as published by \n \
the Free Software Foundation; either version 2 of the License, or \n \
(at your option) any later version."
enum {
        TARGET_INT32,
        TARGET_STRING,
		TARGET_URL,
        TARGET_ROOTWIN
	};
//Main Window		   
GtkWidget *MainWindow;
GtkWidget *cmbStartStop;
GtkWidget *MainFixed;
GtkWidget *cmbEsci;
GtkWidget *lblStatus;
GtkWidget *Tray;
GtkWidget *VpnsGrid;
GtkListStore  *VpnsGridStore; //data container for Vpns grid
GtkWidget *Locked, *LockedTerminal;
GdkPixbuf *LockedRed, *LockedGray, *LockedGreen;

gchar *FileApp;
gchar *DirApp;
gchar *VpnsDir; 
gchar *CurrentVpnDir;

//Log window
GtkWidget *LogWindow;
GtkWidget* LogTextView;
GtkTextBuffer* LogTextBuffer;
GtkWidget* CheckButtonRefresh;
gboolean CheckLog=TRUE;
GtkWidget* LockedLog;

//Connection Progress window
GtkWidget *ConnectionProgressWindow;


//PrivateKey Window
GtkWidget *PrivateKeyWindow;
GtkWidget *EntryPrivateKey;
gchar *PasswordPrivateKey;


//UsernamePasswordWindow 
GtkWidget *UserNamePasswordWindow;
GtkWidget *EntryUser;
GtkWidget *EntryPassword;
gchar *User;
gchar *Password;



const gchar *PIDfile="/tmp/quick_vpn.pid";
const gchar *StatusFile="/tmp/quick_vpn.status";
const gchar *ErrFile="/tmp/quick_vpn.err";//non usato
const gchar *LogFile="/tmp/quick_vpn.log";

//Prototipi
int controlloreVpn();
static void StartStop(GtkWidget *widget, gpointer data);
static GtkWidget *InitAboutWindow();
static void VpnDelete(GtkWidget *widget, gpointer data);
static void
drag_data_received
(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
        GtkSelectionData *selection_data, guint target_type, guint time,
        gpointer data);
static void VpnAdd(GtkWidget *widget, gpointer data);
static void CloseMainWindow( GtkWidget *widget, gpointer   data );
void UpdateVpnGrid(GtkWidget *view);
void SetPasswordPrivateKey();
void InitConnectionProgress();
gboolean ManagementServerIsUp();
gboolean StopVpn();
void SetUserPassword();
gboolean SendUserPassword();
void CloseLogWindow();
void ShowLogWindow();
void RefreshLog();
int UpdateLogTextView();

static GtkItemFactoryEntry menu_items[] = {
  { "/_VPNs",         NULL,      NULL,           0, "<LastBranch>" },
  { "/_VPNs/_Add",    "<CTRL>O", VpnAdd,           0, "<StockItem>",GTK_STOCK_ADD},
  { "/_VPNs/_Delete", NULL,      VpnDelete,           0, "<StockItem>",GTK_STOCK_DELETE},
  { "/_VPNs/_Log window", "<CTRL>L",  ShowLogWindow,           0, "<StockItem>",GTK_STOCK_INDEX},
  { "/_VPNs/Exit",    NULL,      CloseMainWindow,           0, "<StockItem>",GTK_STOCK_QUIT},
  { "/_Help",         NULL,      NULL,           0, "<LastBranch>" },
  { "/_Help/About",   NULL , InitAboutWindow, 0, "<StockItem>",GTK_STOCK_ABOUT}
};

void InitLogWindow()
{
	
	LogWindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(LogWindow),GTK_WIN_POS_CENTER);
	gtk_window_set_icon(GTK_WINDOW(LogWindow),LockedGray);
	gtk_window_set_title(GTK_WINDOW(LogWindow),"Openvpn log :");
	gtk_window_set_resizable(GTK_WINDOW(LogWindow),FALSE);
	
	
	GtkWidget* LogFixed=gtk_fixed_new();
	gtk_widget_set_size_request(GTK_WIDGET(LogFixed),700,480);
	gtk_container_add (GTK_CONTAINER (LogWindow), LogFixed);
	
	LogTextView= gtk_text_view_new();
	
	
	LogTextBuffer=gtk_text_buffer_new(NULL);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(LogTextView),FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(LogTextView),FALSE);
	
	GtkWidget *LogScrollbar=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(LogScrollbar),NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(LogScrollbar),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(LogScrollbar), GTK_WIDGET(LogTextView));
	gtk_widget_set_size_request(GTK_WIDGET(LogScrollbar),700,400);
	gtk_fixed_put(GTK_FIXED(LogFixed),LogScrollbar,1,1);
	
	GtkWidget* ButtonClose=gtk_button_new_from_stock("gtk-close");
	gtk_fixed_put(GTK_FIXED(LogFixed),ButtonClose,290,430);
	
	CheckButtonRefresh=gtk_check_button_new_with_label("Refresh log");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CheckButtonRefresh),TRUE);
	gtk_fixed_put(GTK_FIXED(LogFixed),CheckButtonRefresh,1,410);
	
	LockedLog= gtk_image_new_from_pixbuf(GDK_PIXBUF(LockedGray));
	gtk_fixed_put(GTK_FIXED(LogFixed),LockedLog,640,405);
	
	//Callbacks
	g_signal_connect (G_OBJECT (ButtonClose), "clicked",G_CALLBACK (CloseLogWindow), NULL);
	g_signal_connect (G_OBJECT (LogWindow), "delete-event", G_CALLBACK (CloseLogWindow), NULL);
	g_signal_connect (G_OBJECT (CheckButtonRefresh), "toggled", G_CALLBACK (RefreshLog), NULL);
}


void CloseLogWindow()
{
	CheckLog=FALSE;
	gtk_widget_hide(GTK_WIDGET(LogWindow));
}
void ShowLogWindow()
{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CheckButtonRefresh),TRUE);
	CheckLog=TRUE;
	g_timeout_add(1000, UpdateLogTextView, NULL);
	gtk_widget_show_all(GTK_WIDGET(LogWindow));
	
}

void RefreshLog()
{
	if (CheckLog==TRUE){
		CheckLog=FALSE;
	}else{
		CheckLog=TRUE;
		g_timeout_add(1000, UpdateLogTextView, NULL);
	}
}

int UpdateLogTextView()
{
	gchar* line;
	if (g_file_test (LogFile, G_FILE_TEST_EXISTS )){
		GMappedFile* f= g_mapped_file_new(LogFile,  FALSE,NULL);
		
		gtk_text_buffer_set_text(GTK_TEXT_BUFFER(LogTextBuffer),g_mapped_file_get_contents(f), -1);
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(LogTextView),LogTextBuffer);
		g_mapped_file_free(f);	
		
	}
	if (CheckLog==FALSE){
		  return FALSE ;	
	}else return TRUE;
}


void InitConnectionProgress()
{
	GtkWidget *ConnectionProgressFixed;
	GtkWidget *LabelConnectionProgress;
	GtkWidget *ButtonStop;
	
	ConnectionProgressWindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ConnectionProgressWindow),"Connection in progress...");
	gtk_window_set_resizable(GTK_WINDOW(ConnectionProgressWindow),FALSE);
	gtk_window_set_decorated(GTK_WINDOW(ConnectionProgressWindow),FALSE);
	gtk_window_set_position(GTK_WINDOW(ConnectionProgressWindow),GTK_WIN_POS_CENTER);
	
	ConnectionProgressFixed=gtk_fixed_new();
	gtk_widget_set_size_request(GTK_WIDGET(ConnectionProgressFixed),250,100);
	gtk_container_add (GTK_CONTAINER (ConnectionProgressWindow), ConnectionProgressFixed);
	
	LabelConnectionProgress=gtk_label_new("Connection in progress...");
	gtk_label_set_markup(GTK_LABEL(LabelConnectionProgress),"<b>Connection in progress...</b>");
	gtk_fixed_put(GTK_FIXED(ConnectionProgressFixed),LabelConnectionProgress,45,25);

	ButtonStop=gtk_button_new_from_stock("gtk-stop");
	gtk_fixed_put(GTK_FIXED(ConnectionProgressFixed),ButtonStop,95,55);
	g_signal_connect (G_OBJECT (ButtonStop), "clicked",G_CALLBACK (StopVpn), NULL);
	
}

gboolean StopVpn()
{
	/*if (!ManagementServerIsUp()) sleep(2);
	int sd;
	struct sockaddr_in client;
	struct hostent *hp;
	unsigned short port = 9595;
	char buff[MAXBUF];
	hp = gethostbyname("127.0.0.1");
	
	bzero(&client, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(port);
	client.sin_addr.s_addr =((struct in_addr*)(hp->h_addr))->s_addr;

	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
    		g_print("Socket failed!\n");
			return FALSE;
	}else{
    		g_print("Start socket!\n");
 	}
	if (connect(sd, (struct sockaddr *)&client, sizeof(client)) < 0){
			printf("Error on connection!\n");
		return FALSE;
	}else{
		sprintf(buff,"%s\n","signal SIGTERM");
		send(sd, buff, sizeof(buff), 0);
		close(sd);
		gtk_widget_hide(GTK_WIDGET(ConnectionProgressWindow));
		gtk_widget_hide(GTK_WIDGET(PrivateKeyWindow));
		return TRUE;
	}*/
	//Try to stop VPN
	if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )){
		int pid= PidRead();
		if (kill(pid, SIGTERM)<0){
				//Non e' stato possibile chiudere la Vpn
				err_message(GTK_WINDOW(MainWindow),"Non e\' stato possibile chiudere la connessione!");
			}else{
				//E' stato possibile chiudere la vpn
				if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )) g_unlink(PIDfile);
			}
		}
		gtk_widget_hide(GTK_WIDGET(ConnectionProgressWindow));
		gtk_widget_hide(GTK_WIDGET(PrivateKeyWindow));
		gtk_widget_hide(GTK_WIDGET(UserNamePasswordWindow));
		gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid), TRUE);
}


gboolean ManagementServerIsUp()
{
	int sd;
	struct sockaddr_in client;
	struct hostent *hp;
	unsigned short port = 9595;
	char buff[MAXBUF];
	hp = gethostbyname("127.0.0.1");
	
	bzero(&client, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(port);
	client.sin_addr.s_addr =((struct in_addr*)(hp->h_addr))->s_addr;

	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
    		g_print("Socket failed!\n");
			return FALSE;
	}else{
    		g_print("Start socket!\n");
 	}
	if (connect(sd, (struct sockaddr *)&client, sizeof(client)) < 0){
			printf("Error on connection!\n");
		return FALSE;
	}else{
		close(sd);
		return TRUE;
	}
}

void InitPrivateKeyWindow()
{
	GtkWidget *PrivateKeyFixed;
	GtkWidget *ButtonOk, *ButtonStop;
	GtkWidget *LabelPrivateKey;
	
	PrivateKeyWindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(PrivateKeyWindow),"Password for private key:");
	gtk_window_set_resizable(GTK_WINDOW(PrivateKeyWindow),FALSE);
	gtk_window_set_icon(GTK_WINDOW(PrivateKeyWindow),LockedGray);
	PrivateKeyFixed=gtk_fixed_new();
	gtk_widget_set_size_request(GTK_WIDGET(PrivateKeyFixed),250,200);
	gtk_container_add (GTK_CONTAINER (PrivateKeyWindow), PrivateKeyFixed);

	LabelPrivateKey=gtk_label_new("Enter private key password:");
	EntryPrivateKey=gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(EntryPrivateKey),FALSE);
	ButtonOk=gtk_button_new_from_stock("gtk-ok");
			
	ButtonStop=gtk_button_new_from_stock("gtk-stop");
	
	gtk_fixed_put(GTK_FIXED(PrivateKeyFixed),LabelPrivateKey,50,50);
	gtk_fixed_put(GTK_FIXED(PrivateKeyFixed),EntryPrivateKey,50,80);
	gtk_fixed_put(GTK_FIXED(PrivateKeyFixed),ButtonOk,180,160);
	gtk_fixed_put(GTK_FIXED(PrivateKeyFixed),ButtonStop,105,160);
	
	g_signal_connect (G_OBJECT (PrivateKeyWindow), "delete-event",
	       G_CALLBACK (SetPasswordPrivateKey), NULL);
	
	g_signal_connect (G_OBJECT (ButtonOk), "clicked",
		      G_CALLBACK (SetPasswordPrivateKey), NULL);
	g_signal_connect (G_OBJECT (ButtonStop), "clicked",
			G_CALLBACK (StopVpn), NULL);
	
}



void InitUsernamePasswordWindow()
{
	GtkWidget *UsernamePasswordFixed;
	GtkWidget *ButtonOk, *ButtonStop;
	GtkWidget *LabelUsername, *LabelPassword;
	
	UserNamePasswordWindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(UserNamePasswordWindow),"Insert Usename and password:");
	gtk_window_set_resizable(GTK_WINDOW(UserNamePasswordWindow),FALSE);
	gtk_window_set_icon(GTK_WINDOW(UserNamePasswordWindow),LockedGray);
	
	LabelUsername=gtk_label_new("Username:");
	EntryUser=gtk_entry_new();
	
	LabelPassword=gtk_label_new("Password:");
	EntryPassword=gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(EntryPassword),FALSE);
	
	ButtonOk=gtk_button_new_from_stock("gtk-ok");
	ButtonStop=gtk_button_new_from_stock("gtk-stop");
	
	
	UsernamePasswordFixed=gtk_fixed_new();
	gtk_container_add (GTK_CONTAINER (UserNamePasswordWindow), UsernamePasswordFixed);
	gtk_widget_set_size_request(GTK_WIDGET(UsernamePasswordFixed),250,260);
	
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),LabelUsername,50,50);
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),EntryUser,50,80);
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),LabelPassword,50,130);
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),EntryPassword,50,150);
	
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),ButtonOk,180,210);
	gtk_fixed_put(GTK_FIXED(UsernamePasswordFixed),ButtonStop,105,210);
	
	g_signal_connect (G_OBJECT (UserNamePasswordWindow), "delete-event",
	       G_CALLBACK (StopVpn), NULL);
	g_signal_connect (G_OBJECT (ButtonOk), "clicked",
		      G_CALLBACK (SetUserPassword), NULL);
	g_signal_connect (G_OBJECT (ButtonStop), "clicked",
			G_CALLBACK (StopVpn), NULL);
	
}


void SetPasswordPrivateKey()
{
		PasswordPrivateKey=g_strconcat("password \"Private Key\" ",gtk_entry_get_text(GTK_ENTRY(EntryPrivateKey)),NULL);
		if (strlen(gtk_entry_get_text(GTK_ENTRY(EntryPrivateKey)))>1){
			if (SendKeyPassword()==TRUE){
				gtk_widget_hide(GTK_WIDGET(PrivateKeyWindow));
				gtk_widget_show(GTK_WIDGET(ConnectionProgressWindow));
			}
		}else{
			err_message(GTK_WINDOW(MainWindow),"Password key too short.");
			StopVpn();
		}	
}

void SetUserPassword()
{
	User=g_strconcat("username \"Auth\" ",gtk_entry_get_text(GTK_ENTRY(EntryUser)), NULL);
	Password=g_strconcat("password \"Auth\" ",gtk_entry_get_text(GTK_ENTRY(EntryPassword)),NULL);
	if (SendUserPassword()==TRUE){
		gtk_widget_hide(GTK_WIDGET(UserNamePasswordWindow));
		gtk_widget_show(GTK_WIDGET(ConnectionProgressWindow));
	}
}


gboolean SendKeyPassword()
{
	int sd;
	struct sockaddr_in client;
	struct hostent *hp;
	unsigned short port = 9595;
	char buff[MAXBUF];
	hp = gethostbyname("127.0.0.1");
	
	bzero(&client, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(port);
	client.sin_addr.s_addr =((struct in_addr*)(hp->h_addr))->s_addr;

	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    		g_print("Socket failed!\n");
	else
    		g_print("Start socket!\n");
 
	if (connect(sd, (struct sockaddr *)&client, sizeof(client)) < 0){
    		printf("Error on connection!\n");
		return -1;
	}
	recv(sd, buff, sizeof(buff), 0);
	sprintf(buff,"%s\n",PasswordPrivateKey);
	send(sd, buff, sizeof(buff), 0);
		
	//Check if password is correct, if password key is incorrect openvpn exit
	if (ManagementServerIsUp()==FALSE){
		err_message(GTK_WINDOW(MainWindow),"Passwork key not correct!");
		gtk_widget_hide(GTK_WIDGET(PrivateKeyWindow));
		gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),TRUE);
		close(sd);
		return FALSE;
	}else{
		close(sd);
		return TRUE;
	}
	close(sd);
}


gboolean SendUserPassword()
{
	int sd;
	struct sockaddr_in client;
	struct hostent *hp;
	unsigned short port = 9595;
	char buff[MAXBUF];
	hp = gethostbyname("127.0.0.1");
	
	bzero(&client, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(port);
	client.sin_addr.s_addr =((struct in_addr*)(hp->h_addr))->s_addr;

	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    		g_print("Socket failed!\n");
	else
    		g_print("Start socket!\n");
 
	if (connect(sd, (struct sockaddr *)&client, sizeof(client)) < 0){
    		g_print("Error on connection!\n");
			return -1;
	}
	
	//send user
	recv(sd, buff, sizeof(buff), 0);
	sprintf(buff,"%s\n",User);
	send(sd, buff, sizeof(buff), 0);
	//debug
	g_print(User);
	
	//send passsword
	//recv(sd, buff, sizeof(buff), 0);
	sprintf(buff,"%s\n",Password);
	send(sd, buff, sizeof(buff), 0);
	//debug
	g_print(Password);
		
	//Check if password is correct, if password key is incorrect openvpn exit
	if (ManagementServerIsUp()==FALSE){
		err_message(GTK_WINDOW(MainWindow),"User or Passwork are not correct!");
		gtk_widget_hide(GTK_WIDGET(UserNamePasswordWindow));
		gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),TRUE);
		close(sd);
		return FALSE;
	}else{
		close(sd);
		return TRUE;
	}
	close(sd);
}


gboolean CheckForPassword(){
	int sd;
	struct sockaddr_in client;
	struct hostent *hp;
	unsigned short port = 9595;
	char buff[MAXBUF];
	hp = gethostbyname("127.0.0.1");
	
	bzero(&client, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(port);
	client.sin_addr.s_addr =((struct in_addr*)(hp->h_addr))->s_addr;

	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    		g_print("Socket failed!\n");
	else
    		g_print("Start socket!\n");
 
	if (connect(sd, (struct sockaddr *)&client, sizeof(client)) < 0){
			printf("Error on connection!\n");
		return FALSE;
	}
	recv(sd, buff, sizeof(buff), 0);
	g_print("Buffer ricevuto ->%s\n",buff);
	
	//ask for private key password
	gtk_widget_hide(GTK_WIDGET(PrivateKeyWindow));
	if (g_strrstr(buff,">PASSWORD:Need 'Private Key' password")!=NULL)
	{
		close(sd);
		gtk_entry_set_text(GTK_ENTRY(EntryPrivateKey),"");
		PasswordPrivateKey=g_strdup("");
		gtk_widget_show_all(GTK_WIDGET(PrivateKeyWindow));
		gtk_widget_hide(GTK_WIDGET(ConnectionProgressWindow));
		return FALSE;
	}
	//ask for username and password
	if (g_strrstr(buff,">PASSWORD:Need 'Auth' username/password")!=NULL)
	{
		//code for ask username and password
		close(sd);
		gtk_entry_set_text(GTK_ENTRY(EntryPassword),"");
		Password=g_strdup("");
		gtk_widget_show_all(GTK_WIDGET(UserNamePasswordWindow));
		
		//after username/password can ask for privatekey password
		if (g_strrstr(buff,">PASSWORD:Need 'Private Key' password")!=NULL)
		{
			close(sd);
			gtk_entry_set_text(GTK_ENTRY(EntryPrivateKey),"");
			PasswordPrivateKey=g_strdup("");
			gtk_widget_show_all(GTK_WIDGET(PrivateKeyWindow));
			gtk_widget_hide(GTK_WIDGET(ConnectionProgressWindow));
		}
		return FALSE;
	}
		
	//then exit management session
	close(sd);
	return TRUE;
}


static void VpnAdd(GtkWidget *widget, gpointer data)
{
	
	GtkWidget *dialog;
	GtkFileFilter* filter;
	
	
	dialog = gtk_file_chooser_dialog_new ("Select a folder",
						  GTK_WINDOW(MainWindow),
						  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
						  NULL);
	//Filter for directory
	filter=gtk_file_filter_new();
	gtk_file_filter_add_mime_type(GTK_FILE_FILTER(filter),"inode/directory");
	gtk_file_filter_set_name(GTK_FILE_FILTER(filter),"Directory");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog),filter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	  {
		gchar *filename, *basename, *destlink;
		
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		basename=g_filename_display_basename(filename);
		destlink=g_build_filename(VpnsDir,basename,NULL);
		
		GFile *LinkFile=g_file_new_for_path(destlink); 
		
		if(g_file_make_symbolic_link(LinkFile, filename, NULL,NULL )==TRUE)
		{
			UpdateVpnGrid(VpnsGrid);
		}else{
			err_message(GTK_WINDOW(MainWindow), "Sorry, I can't make a link for you Vpn.");
		}
		g_free (filename);
	  }
	gtk_widget_destroy (dialog);
}

static void VpnDelete(GtkWidget *widget, gpointer data)
{
	if (info_YesNo(GTK_WINDOW(MainWindow),"Do you want really delete this Vpn's file?"  )==GTK_RESPONSE_YES)
	{
		 if (g_file_test (CurrentVpnDir, G_FILE_TEST_IS_SYMLINK)){
		 	if (g_unlink(CurrentVpnDir)!=0) 
					err_message(GTK_WINDOW(MainWindow),"Error on delete file!");
		 }
		 if (g_file_test (CurrentVpnDir,G_FILE_TEST_IS_DIR)){
			 if (g_rmdir(CurrentVpnDir)!=0)
			 	err_message(GTK_WINDOW(MainWindow),"Error on delete folder!");
		}
		UpdateVpnGrid(VpnsGrid);
	}
}


static GtkWidget *get_menubar_menu( GtkWidget  *window )
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  /* Make an accelerator group (shortcut keys) */
  accel_group = gtk_accel_group_new ();

  /* Make an ItemFactory (that makes a menubar) */
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
	static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* Finally, return the actual menu bar created by the item factory. */
  return gtk_item_factory_get_widget (item_factory, "<main>");
}

static GtkWidget *InitAboutWindow()
{
	const gchar *authors[]={
		("Fabio Di Matteo - pobfdm@gmail.com"), ("And you? - your@email.com")
		,NULL};
	
	const gchar *artists[]={
		("Alexandre Moore http://www.gnome-look.org/content/show.php?content=28352"), ("And you? - your@email.com")
		,NULL};
	
	
	GtkWidget *AboutWindow= gtk_about_dialog_new();
	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(AboutWindow),VERSION);
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(AboutWindow),authors);
	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(AboutWindow),artists);
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(AboutWindow),LICENSE);
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(AboutWindow),("http://www.freemedialab.org/quickvpn"));
	gtk_dialog_run(GTK_DIALOG(AboutWindow));
	gtk_widget_destroy(GTK_WIDGET(AboutWindow));
	
	
}



int PidRead(){
		char pid[6];
		if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )){
			FILE *f=fopen(PIDfile, "r");
			fgets(pid,6,f);
			fclose(f);
		}
		return atoi(pid);
}



void PuliziaFile(){
	if (g_file_test (PIDfile, G_FILE_TEST_EXISTS )) g_unlink(PIDfile);
    if (g_file_test (ErrFile, G_FILE_TEST_EXISTS )) g_unlink(ErrFile);	
    if (g_file_test (StatusFile, G_FILE_TEST_EXISTS )) g_unlink(StatusFile);
    if (g_file_test (LogFile, G_FILE_TEST_EXISTS )) g_unlink(LogFile);
}


	
gboolean VpnIsStart(){
		char buff;
		gboolean res=FALSE;
		if (g_file_test (StatusFile, G_FILE_TEST_EXISTS )==TRUE){
			FILE *f=fopen(StatusFile, "r");
			buff=fgetc(f);
			fclose(f);
			if (buff=='1'){
				res=TRUE;
			}else{
				res=FALSE;
			}
		}else{
			res=FALSE;
		}	
			
		return res;	
}


static void CloseMainWindow( GtkWidget *widget, gpointer   data )
{
    StopVpn();
    PuliziaFile();
    gtk_main_quit ();
}


static void StartStop(GtkWidget *widget, gpointer data)
{
	//If present use OpenVpn binary included on quickvpn
	gchar *OpenVpnBin=g_build_filename(DirApp,"openvpn",NULL);
	if (g_file_test (OpenVpnBin, G_FILE_TEST_EXISTS )==FALSE) 
									OpenVpnBin=g_strdup("openvpn");
	
	gchar *CmdStart=g_strconcat(OpenVpnBin," --management 127.0.0.1 9595 --management-query-passwords  --verb 3 --log ",LogFile,
	" --daemon --config "	,CurrentVpnDir,"/openvpn.conf --writepid ",
	PIDfile, " --cd ", CurrentVpnDir, " --up 'echo 1 > ",StatusFile,
	"' --down 'echo 0 > ",StatusFile , " '", NULL);
	
		
	//debug
	g_print("Pid file -> %s\n",PIDfile);
	g_print("Open VPN -> %s\n",OpenVpnBin);
	g_print("CurrentVpnDir -> %s\n",CurrentVpnDir);
	g_print("Comando -> %s\n",CmdStart);
	
	//Hide main window...
	//gtk_widget_hide(GTK_WIDGET(MainWindow));
	
	
	//Check if openvpn is started
	if(VpnIsStart()==FALSE){
		if (g_spawn_command_line_async(CmdStart,NULL)==FALSE){
			err_message(GTK_WINDOW(MainWindow)," Sorry, i can't find your openvpn");
		}else{
			gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid), FALSE);
			gtk_widget_show_all(GTK_WIDGET(ConnectionProgressWindow));
			g_timeout_add(1000, CheckForPassword, NULL);
		}
	}else{
		StopVpn();
	}
	
}

static void TrayClick(GtkWidget *widget, gpointer data)
{
	if ( gtk_window_is_active(GTK_WINDOW(MainWindow))==TRUE ){
		gtk_widget_hide(GTK_WIDGET(MainWindow));
	}else{
		gtk_widget_show_all(GTK_WIDGET(MainWindow));
		gtk_window_deiconify(GTK_WINDOW(MainWindow));
	}
}

static void HideMainWindow(GtkWidget *widget, gpointer data)
{
	gtk_widget_hide(GTK_WIDGET(MainWindow));
}


int controlloreVpn()
{
		if (VpnIsStart()==TRUE){
			gtk_button_set_label(GTK_BUTTON(cmbStartStop),"gtk-disconnect");
			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(Tray),GDK_PIXBUF(LockedGreen));
			gtk_status_icon_set_tooltip(GTK_STATUS_ICON(Tray), "Vpn connessa.");
			gtk_image_set_from_pixbuf(GTK_IMAGE(Locked),GDK_PIXBUF(LockedGreen));
			gtk_image_set_from_pixbuf(GTK_IMAGE(LockedLog),GDK_PIXBUF(LockedGreen));
			//gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),FALSE);
			gtk_widget_hide(GTK_WIDGET(ConnectionProgressWindow));
		}else{
			gtk_button_set_label(GTK_BUTTON(cmbStartStop),"gtk-connect");
			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(Tray),GDK_PIXBUF(LockedRed));
			gtk_status_icon_set_tooltip(GTK_STATUS_ICON(Tray), "Vpn non connessa.");
			gtk_image_set_from_pixbuf(GTK_IMAGE(Locked),GDK_PIXBUF(LockedRed));
			gtk_image_set_from_pixbuf(GTK_IMAGE(LockedLog),GDK_PIXBUF(LockedRed));
			//gtk_widget_set_sensitive(GTK_WIDGET(VpnsGrid),TRUE);
		}
	return TRUE ;
}


void UpdateVpnGrid(GtkWidget *view)
{
	enum { COL_IMG  , COL_VPN, NUM_COLS } ;
	/*Container not visible*/
    GtkTreeIter    iter;
  	GdkPixbuf     *pixbuf;
	
	
	if (g_file_test (g_build_filename(DirApp,"/img/locked_32.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
	pixbuf=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_32.png",NULL), NULL);
	else pixbuf=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"../share/quickvpn/locked_32.png",NULL), NULL);
	

	/*Visual Container*/
	GtkCellRenderer     *renderer; //la cella
	
	gtk_list_store_clear(GTK_LIST_STORE(VpnsGridStore));
	
	/*Scan Vpn dir*/
	struct dirent **namelist; 
	int n, i=1; 
	 
	n = scandir(VpnsDir, &namelist, 0, alphasort); 
	if (n < 0){ 
		perror("No vpn found!\n"); 
	}else{
		while(n--){
			if (strcmp(namelist[n]->d_name,".")!=0 && strcmp(namelist[n]->d_name,"..")!=0){
				/* Add a row */
				gtk_list_store_append (VpnsGridStore, &iter);
				gtk_list_store_set (VpnsGridStore, &iter,COL_IMG, pixbuf, COL_VPN, namelist[n]->d_name,-1);
			}		
		}
	}
	
  GtkTreeModel  *ModelUpdate=GTK_TREE_MODEL (VpnsGridStore); 
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), ModelUpdate);
}

GtkWidget *InitVpnGrid()
{
	enum { COL_IMG  , COL_VPN, NUM_COLS } ;
	
	/*Container not visible*/
	
    GtkTreeIter    iter;  //segnaposto griglia
  	GdkPixbuf     *pixbuf;
	
	if (g_file_test (g_build_filename(DirApp,"/img/locked_32.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
	pixbuf=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_32.png",NULL), NULL);
	else pixbuf=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"../share/quickvpn/locked_32.png",NULL), NULL);

	/*Visual Container*/
	GtkCellRenderer     *renderer; //la cella
  	GtkWidget           *view;
	view = gtk_tree_view_new();
	
	
	//Creo il contenitore dei dati, specificando il tipo per ogni colonna
  	VpnsGridStore = gtk_list_store_new (NUM_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	
	/*Scan Vpn dir*/
	struct dirent **namelist; 
	int n, i=1; 
	 
	n = scandir(VpnsDir, &namelist, 0, alphasort); 
	if (n < 0){ 
		perror("No vpn found!\n"); 
	}else{
		while(n--){
			if (strcmp(namelist[n]->d_name,".")!=0 && strcmp(namelist[n]->d_name,"..")!=0){
				/* Aggiungiamo una riga */
				gtk_list_store_append (VpnsGridStore, &iter);
				gtk_list_store_set (VpnsGridStore, &iter,COL_IMG, pixbuf, COL_VPN, namelist[n]->d_name,-1);
			}		
		}
	}
	
	
   /* --- C1 --- */
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,      
                                               "",  
                                               renderer,
                                               "pixbuf", COL_IMG,
                                               NULL);
	/* --- C2 --- */

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
                                               -1,      
                                               "VPNs",  
                                               renderer,
                                               "text", COL_VPN,
                                               NULL);
											   
  GtkTreeModel  *model=GTK_TREE_MODEL (VpnsGridStore);
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), model);
  g_object_unref (model);   
  
  //For Drad and Drop
  //http://live.gnome.org/GnomeLove/DragNDropTutorial
  
  static GtkTargetEntry target_list[] = {
        //{ "INTEGER",    0, TARGET_INT32 },
        //{ "STRING",     0, TARGET_STRING },
        //{ "text/plain", 0, TARGET_STRING },
		{ "text/uri-list", 0, TARGET_URL },
        //{ "application/x-rootwindow-drop", 0, TARGET_ROOTWIN }
	};
 static guint n_targets = G_N_ELEMENTS (target_list);
 gtk_drag_dest_set(GTK_WIDGET(view), GTK_DEST_DEFAULT_ALL, target_list,n_targets,GDK_ACTION_COPY);

  return GTK_WIDGET (view);

}

gboolean GridVpnsCursorChanged (GtkTreeView *tree_view, gpointer user_data) 
{
	GtkTreeIter   iter;
  	GtkTreeModel *model;
	GtkTreeSelection *selection;
	gchar *VPN, *labelVPN;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	model = gtk_tree_view_get_model(tree_view);
  	
	if (gtk_tree_selection_get_selected(selection, &model, &iter))
  	{
		gtk_tree_model_get(model, &iter, 1, &VPN, -1);
		//Only for label
		labelVPN = g_markup_printf_escaped("<b>%s</b>", VPN);
    	gtk_label_set_markup(GTK_LABEL(lblStatus),labelVPN);
		
		g_print ("CurrentVpn->%s\n", VPN);
    	CurrentVpnDir=g_build_filename(VpnsDir,VPN,NULL);
		g_free(VPN);
  	}
	gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),TRUE);
	
}


void
onGridVpnsRowChanged (GtkTreeView *view, GtkTreePath *path,
                        GtkTreeViewColumn *col, gpointer userdata)
{
 	GtkTreeIter   iter;
  	GtkTreeModel *model;
	gchar *VPN, *labelVPN;
	
  	model = gtk_tree_view_get_model(view);

  if (gtk_tree_model_get_iter(model, &iter, path))
  {
    	gtk_tree_model_get(model, &iter, 1, &VPN, -1);
		
		//Only for label
		labelVPN = g_markup_printf_escaped("<b>%s</b>", VPN);
    	gtk_label_set_markup(GTK_LABEL(lblStatus),labelVPN);
		
		g_print ("CurrentVpn->%s\n", VPN);
    	CurrentVpnDir=g_build_filename(VpnsDir,VPN,NULL);
		g_free(VPN);
		StartStop(NULL,NULL);
  }
	gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),TRUE);
  
}




void InitMainWindow()
{
if (getuid()==0){	
	//Finestra principale
	MainWindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (MainWindow), VERSION );
	gtk_window_set_resizable(GTK_WINDOW(MainWindow),1);
	gtk_window_set_position(GTK_WINDOW(MainWindow),GTK_WIN_POS_MOUSE);
	//gtk_window_set_default_size(GTK_WINDOW(MainWindow), 200, 450);
	gtk_window_set_decorated(GTK_WINDOW(MainWindow),TRUE);
	gtk_window_set_resizable(GTK_WINDOW(MainWindow),FALSE);
	if (g_file_test (g_build_filename(DirApp,"/img/locked_gray.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
		gtk_window_set_icon_from_file(GTK_WINDOW(MainWindow),g_build_filename(DirApp,"/img/locked_gray.png",NULL),NULL);
	else gtk_window_set_icon_from_file(GTK_WINDOW(MainWindow),g_build_filename(DirApp,"../share/quickvpn/locked_gray.png",NULL),NULL);
	
	//Vpns Grid
	VpnsGrid =InitVpnGrid();
	GtkWidget *GridScrollbar=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(GridScrollbar),NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(GridScrollbar),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(GridScrollbar), GTK_WIDGET(VpnsGrid));
	gtk_widget_set_size_request(GTK_WIDGET(GridScrollbar),190,200);
	
	
	//Lockeds
	if (g_file_test (g_build_filename(DirApp,"/img/locked_gray.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
	LockedGray=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_gray.png",NULL), NULL);
	else LockedGray=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"../share/quickvpn/locked_gray.png",NULL), NULL);
	
	if (g_file_test (g_build_filename(DirApp,"/img/locked_green.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
	LockedGreen=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_green.png",NULL), NULL);
	else LockedGreen=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"../share/quickvpn/locked_green.png",NULL), NULL);
	
	if (g_file_test (g_build_filename(DirApp,"/img/locked_red.png",NULL), G_FILE_TEST_EXISTS )==TRUE)
	LockedRed=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"/img/locked_red.png",NULL), NULL);
	else LockedRed=gdk_pixbuf_new_from_file(g_build_filename(DirApp,"../share/quickvpn/locked_red.png",NULL), NULL);
	
	//Big Image locked
	Locked=gtk_image_new_from_pixbuf(GDK_PIXBUF(LockedGray));
	
	//Tray icon
	Tray=gtk_status_icon_new_from_pixbuf(GDK_PIXBUF(LockedGray));
	
	//Pulsanti
	cmbStartStop=gtk_button_new_from_stock("gtk-connect");
	//gtk_widget_set_sensitive(GTK_WIDGET(cmbStartStop),FALSE);
	lblStatus=gtk_label_new("<b>Select a vpn...</b>");
	 gtk_label_set_justify(GTK_LABEL(lblStatus),GTK_JUSTIFY_CENTER);
	gtk_label_set_use_markup(GTK_LABEL(lblStatus),1);
	cmbEsci=gtk_button_new_from_stock("gtk-quit");
	
	//Container for MainWindow
	MainFixed=gtk_fixed_new();
	gtk_widget_set_size_request(GTK_WIDGET(MainFixed),200,450);
	gtk_container_add (GTK_CONTAINER (MainWindow), MainFixed);
  	
	//Menubar 
	GtkWidget *MenuBar=gtk_menu_bar_new();
	MenuBar = get_menubar_menu (MainWindow);
	
	
  	//Aggiungiamo i widgets al contenitore
	gtk_fixed_put(GTK_FIXED(MainFixed),MenuBar,1,1);
  	gtk_fixed_put(GTK_FIXED(MainFixed),cmbStartStop,55,340);
  	gtk_fixed_put(GTK_FIXED(MainFixed),cmbEsci,70,400);
  	gtk_fixed_put(GTK_FIXED(MainFixed),lblStatus,95,270);
  	gtk_fixed_put(GTK_FIXED(MainFixed),GridScrollbar,5,35);
	gtk_fixed_put(GTK_FIXED(MainFixed),Locked, 30,245);
  	
	//Mostriamo la finestra principale ed il suo contenuto
  	//gtk_widget_show_all(GTK_WIDGET(MainWindow));
	
	
	//Connettiamo le callbacks
	g_signal_connect (G_OBJECT (MainWindow), "delete-event",
		      G_CALLBACK (HideMainWindow), NULL);
	g_signal_connect (G_OBJECT (cmbEsci), "clicked",
		      G_CALLBACK (CloseMainWindow), NULL);
	g_signal_connect (G_OBJECT (cmbStartStop), "clicked",
		      G_CALLBACK (StartStop), NULL);
	g_signal_connect (G_OBJECT (Tray), "activate",
		      G_CALLBACK (TrayClick), NULL);
	g_signal_connect (G_OBJECT (VpnsGrid), "row-activated",
		      G_CALLBACK (onGridVpnsRowChanged), NULL);	
	g_signal_connect (G_OBJECT (VpnsGrid), "cursor-changed",
		      G_CALLBACK (GridVpnsCursorChanged), NULL);		  
	g_signal_connect (G_OBJECT (VpnsGrid), "drag-data-received",
		      G_CALLBACK (drag_data_received), NULL);		  
	
	InitPrivateKeyWindow();
	InitConnectionProgress();
	InitUsernamePasswordWindow();
	InitLogWindow();
}else{
	err_message(GTK_WINDOW(MainWindow),"Sorry, but you are not root.");
	gtk_main_quit();
	exit(-1);
}	
	
}	



static void
drag_data_received
(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
        GtkSelectionData *selection_data, guint target_type, guint time,
        gpointer data)
{
	if((selection_data != NULL) && (selection_data-> length >= 0))
    {
         if(target_type==TARGET_URL)
		 {
		 		gchar *uri = (guchar*)selection_data->data;
				gchar *dirname=g_filename_from_uri(uri,NULL,NULL);
				dirname=g_filename_to_utf8(dirname,-1,NULL,NULL,NULL);
								
				gchar *dirbasename=g_filename_display_basename(dirname);
				gchar *destlink=g_build_filename(VpnsDir,dirbasename,NULL);
				
								
				g_print ("Dirname: %s\n", dirname);
				g_print ("Destlink: %s\n", destlink);
												
				gchar *cmd=g_strconcat("cp -Rf ",dirname," ",destlink,NULL);
				g_print("Comando ->%s\n",cmd);
				info_message(GTK_WINDOW(MainWindow),"Help me with this drag and drop!\n");
				
				/*GFile *LinkFile=g_file_new_for_path(destlink); 
				g_file_make_symbolic_link(LinkFile,
										dirname, NULL,NULL );*/
				//system(cmd);
				
				
				/*if (symlink(dirname,destlink)!=0)
				{
					err_message(GTK_WINDOW(MainWindow),"Error!");
				}*/
				
		 }
	}

}



int main(int argc, char* argv[])
{
	if (g_find_program_in_path ("quickvpn")==NULL ){
		//if quickvpn run in local folder
		FileApp=argv[0];
		DirApp = g_path_get_dirname(argv[0]);
		VpnsDir=g_build_filename(DirApp,"VPNs/",NULL);
	}else{
		//if quickvpn is installed on system
		DirApp = g_path_get_dirname(g_find_program_in_path("quickvpn"));
		VpnsDir=g_build_filename(g_get_home_dir(),".quickvpn/VPNs/",NULL);
		
		//Make a dir for store settings
		if (g_file_test (VpnsDir, G_FILE_TEST_EXISTS )==FALSE){
			if (g_file_test (VpnsDir, G_FILE_TEST_IS_DIR)==FALSE) {
 				g_mkdir(g_build_filename(g_get_home_dir(),".quickvpn",NULL), 700); 
				g_mkdir(g_build_filename(g_get_home_dir(),".quickvpn/VPNs",NULL), 700); 
					//err_message(GTK_WINDOW(MainWindow), "Sorry, i can't store settings folder."); 		
			}
		}
	}
			 
	if (argv[1]!=NULL)	VpnsDir=g_build_filename(argv[1], NULL );
	
	
	//Debug
	g_print("App Dir ->%s\n",DirApp);
	g_print("VpnsDir ->%s\n",VpnsDir);	
	PuliziaFile();
		
	gtk_init (&argc, &argv);
	
	//A timer for check connection status
	g_timeout_add(1000, controlloreVpn, NULL);
	
	//A timer for check Log status
	g_timeout_add(1000, UpdateLogTextView, NULL);
			
	//Draw MainWindow
	InitMainWindow();
	
	//Draw TerminalWindow
	//InitTerminalWindow();
	
	
	gtk_main ();
	return 0;
}
